/*
<abstractTopology.h>
Copyright (C) DOLPHIN Project-Team, INRIA Lille - Nord Europe, 2006-2012

Alexandre Quemy, Thibault Lasnier - INSA Rouen

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can  ue,
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.
The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

ParadisEO WebSite : http://paradiseo.gforge.inria.fr
Contact: paradiseo-help@lists.gforge.inria.fr
*/

#ifndef ABSTRACT_TOPO_H_
#define ABSTRACT_TOPO_H_

#include <vector>

namespace paradiseo
{
namespace smp
{

/** AbstractTopology: The Abstract Topology defines what is necessary to any topology.
In order to know a topology, we just need to know how to construct it and what are the neighbors of each node, regardless of the type of topology (boolean or stochastic).

@see smp::topology::Topology, smp::topology::TopologyBuilder

*/

class AbstractTopology
{
public :    

    /**
    * Return a vector containing the index of nearby nodes according to the topology
    * @param idNode index of the node of which you want the neighbors.
    */
	virtual std::vector<unsigned> getIdNeighbors(unsigned idIsland) const = 0;
	
	/**
	* Construct or re-construct a topology with the given number of nodes.
	* @param nbNode number of nodes for the topology 
	*/
	virtual void construct(unsigned nbNode) = 0;

    /**
    *Changes the topology : removes any connection from/to the given node.
    *@param idNode index of the node to be isolated
    */
    virtual void isolateNode(unsigned idNode) = 0;
};

}

}

#endif
